/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.properties;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.AQModelBezeichnung;
import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.doeditor.model.DoeditorPackage;
import de.bsvrz.sys.funclib.bitctrl.modell.bitctrlallgemein.objekte.PortableBitMap;

 
/**
 * Sektion zur Darstellung der Hintergrundeigenschaften einer Flchenform innerhalb einer TabbedPropertySheetPage.
 * 
 * @author Pittner
 * @param <T>
 */
public class BackgroundSection extends AbstractDobjSection<AQModelBezeichnung>
{

  /**
   * Prsentation eines Fllmusters innerhalb der Auswahlliste zur Festlegung des Fllmusters.
   */
  private static class PatternBeschreibung
  {

    /** der Name der Musters. */
    private final String name;

    /**
     * die Bitmap, die das Fllmuster reprsentiert, kann <code>null</code> sein fr "Kein Muster".
     */
    private final PortableBitMap muster;

    /** die statische Beschreibung fr "KeinMuster". */
    private static final PatternBeschreibung KEIN_MUSTER = new PatternBeschreibung("Kein Muster");

    /**
     * Konstruktor, legt eine Fllmusterbeschreibung fr das bergebenen Fllmusterobjekt an.
     * 
     * @param muster
     *          das Musterobjekt
     */
    public PatternBeschreibung(final PortableBitMap muster)
    {
      name = muster.getName();
      this.muster = muster;
    }

    /**
     * Konstruktor, legt eine Musterbeschreibung mit dem bergebenen Name an.
     * 
     * @param name
     *          der Name
     */
    private PatternBeschreibung(final String name)
    {
      this.name = name;
      muster = null;
    }

    @Override
    public boolean equals(final Object obj)
    {

      if (this == obj)
      {
        return true;
      }
      if (obj == null)
      {
        return false;
      }

      if (!(obj instanceof PatternBeschreibung))
      {
        return false;
      }

      final PatternBeschreibung other = (PatternBeschreibung)obj;
      if (muster == null)
      {
        if (other.muster != null)
        {
          return false;
        }
      }
      else if (other.muster == null)
      {
        if (muster != null)
        {
          return false;
        }
      }
      else if (!muster.getPid().equals(other.muster.getPid()))
      {
        return false;
      }
      return true;
    }

    /**
     * liefert das Fllmusterobjekt, mit dem das Fllmuster in der Datenverteilerkonfiguration reprsentiert ist. Die
     * Funmtion kann null liefern, wenn kein Fllmuster definiert werden soll.
     * 
     * @return das Fllmusterobjekt oder <code>null</code>
     */
    public PortableBitMap getMuster()
    {
      return muster;
    }

    @Override
    public int hashCode()
    {
      final int prime = 31;
      int result = 1;
      result = prime * result + (muster == null ? 0 : muster.hashCode());
      return result;
    }

    @Override
    public String toString()
    {
      return name;
    }
  }

  /** das Textfeld zur Anzeige der ausgewhlten Hintergrundfarbe. */
  private Text colorText;

  /** Hintergrundfarbe des Textfeldes. */
  private Color color;

  /** Textfarbe des Textfeldes. */
  private Color textColor;

  /** Schaltflche zum ffnen des Farbauswahldialogs fr die Hintergrundfarbe. */
  private Button colorButton;

  /**
   * Schalter zur Festlegung, ob das betroffene Element gefllt werde soll oder nicht.
   */
  private Button filledButton;

  /**
   * Auswahlliste zur Auswahl des Fllmusters fr die Fllung des Hintergrundes.
   */
  private ComboViewer patternAuswahl;

  @Override
  protected void aktualisiereDaten()
  {
    disposeColors();

    // System.out.println("BackgroundSection.aktualisiereDaten()");

    final RGB rgb = (RGB)getElement().getBackground();

    if (rgb != null)
    {
      final RGB opposite = new RGB(255 - rgb.red, 255 - rgb.green, 255 - rgb.blue);
      color = new Color(Display.getDefault(), rgb);
      textColor = new Color(Display.getDefault(), opposite);
      colorText.setText(color.toString());
      colorText.setBackground(color);
      colorText.setForeground(textColor);
    }
  }

  @Override
  public void createUI(final Composite panel)
  {

    CLabel label = getWidgetFactory().createCLabel(panel, "Hintergrund:");
    FormData data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.right = new FormAttachment(0, AbstractDobjSection.STANDARD_LABEL_WIDTH);
    data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
    label.setLayoutData(data);

    filledButton = getWidgetFactory().createButton(panel, "Gefllt", SWT.CHECK);
    data = new FormData();
    data.left = new FormAttachment(50, 0);
    data.top = new FormAttachment(label, 0, SWT.CENTER);
    filledButton.setLayoutData(data);

    colorButton = getWidgetFactory().createButton(panel, "...", SWT.PUSH);
    data = new FormData();
    data.right = new FormAttachment(filledButton, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(colorText, 0, SWT.CENTER);
    colorButton.setLayoutData(data);

    colorText = getWidgetFactory().createText(panel, "");
    data = new FormData();
    data.left = new FormAttachment(label, 0);
    data.right = new FormAttachment(colorButton, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(label, 0, SWT.CENTER);
    colorText.setLayoutData(data);
    colorText.setEditable(false);

    label = getWidgetFactory().createCLabel(panel, "Fllmuster:");
    data = new FormData();
    data.left = new FormAttachment(filledButton, ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(filledButton, 0, SWT.CENTER);
    label.setLayoutData(data);

    final CCombo combo = getWidgetFactory().createCCombo(panel);
    data = new FormData();
    data.left = new FormAttachment(label, ITabbedPropertyConstants.HSPACE);
    data.right = new FormAttachment(100, 0);
    data.top = new FormAttachment(label, 0, SWT.CENTER);
    combo.setLayoutData(data);

    patternAuswahl = new ComboViewer(combo);
    patternAuswahl.setContentProvider(new ArrayContentProvider());
  }

  @Override
  public void dispose()
  {
    disposeColors();
    super.dispose();
  }

  /** gibt die angelegten Farbressourcen frei. */
  private void disposeColors()
  {
    if (color != null)
    {
      color.dispose();
    }
    if (textColor != null)
    {
      textColor.dispose();
    }
  }

  @Override
  protected void installListeners()
  {     
    colorButton.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(final SelectionEvent e)
      {
        if (isAktiv())
        {
          final ColorDialog dialog = new ColorDialog(getPart().getSite().getShell());

          dialog.setRGB((RGB)getElement().getBackground());
          final RGB newRGB = dialog.open();
          if (newRGB != null)
          {

            getElement().setBackground(newRGB);

            final SetCommand cmd = new SetCommand(getElement(),
                AnlagendarstellungPackage.Literals.AQ_MODEL_BEZEICHNUNG__BACKGROUND, newRGB);
            getCommandStack().execute(cmd);

          }
        }
      }
    });

    filledButton.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(final SelectionEvent e)
      {
        widgetSelected(e);
      }

      @Override
      public void widgetSelected(final SelectionEvent e)
      {
        if (isAktiv())
        {
          final SetCommand cmd = new SetCommand(getElement(), DoeditorPackage.Literals.FLAECHEN_FORM__FILLED,
              filledButton.getSelection());
          getCommandStack().execute(cmd);
        }
      }
    });
  }
}
