package de.bsvrz.buv.plugin.anlagendarstellung.figures;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.MouseEvent;
import org.eclipse.draw2d.MouseListener;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.draw2d.MouseMotionListener;

/**
 * Diese Klasse beinhaltet Hilfsmethoden fr das Drag'n'Drop einer Figure, die
 * Optimierung erfolgte insbesondere fr die Klasse {@link FigurePickerVertical}
 * .<br>
 * 
 * @author Swarco Traffic Systems GmbH
 * 
 */
public class DndHelper // extends MouseMotionListener//MouseMotionListener.Stub
		implements
			MouseMotionListener,
			MouseListener
{
	private Point start;

	/**
	 * Dieses Flag legt fest, ob nur in Y-Richtung verschoben werden darf.
	 */
	private boolean _moveOnlyAlongYAxis = false;

	private boolean _allowMoveOutOfParentsYBounds = false;

	/**
	 * Speichert die eigene Figure, die z.B. verschoben werden soll.
	 */
	private IFigure _figureOwn = null;

	/**
	 * Speichert eine andere Figure zum Zwecke der Kollisionsdetektierung.
	 */
	private IFigure _otherFigure = null;

	private ECollisionLocation _collisionWithOtherFigure = null;

	/**
	 * Listenermechanismus dieser Klasse.
	 */
	public PropertyChangeSupport _pcs = new PropertyChangeSupport(this);

	public static final String PROP_VERSCHOBEN = "VERSCHOBEN";

	/**
	 * 
	 * @param figure
	 *          Die per Drag'n'Drop zu verschiebende Figure.
	 */
	public DndHelper(IFigure figure)
	{
		if (figure != null)
		{
			_figureOwn = figure;

			figure.addMouseMotionListener(this);
			figure.addMouseListener(this);
		}
	}

	/**
	 * Benachrichtigt angemeldete Listener ber eine Positionsnderung via Drag.
	 * 
	 * @param boundsOld
	 *          Alte Position
	 * @param boundsNew
	 *          Neue Position
	 */
	private void fireEventBoundsChange(Rectangle boundsNew)
	{
		_pcs.firePropertyChange(PROP_VERSCHOBEN, null, boundsNew);
	}

	public void mouseDragged(MouseEvent e)
	{
		Point p = e.getLocation();
		Dimension d = p.getDifference(start);
		start = p;
		Figure f = ((Figure) e.getSource());

		/*
		 * Sicherstellen, dass man sich nur entlang der Y-Achse bewegen kann.
		 */
		Rectangle translated = null;
		if (isMoveOnlyAlongYAxis())
		{
			translated = f.getBounds().getTranslated(0, d.height);
		} else
		{
			translated = f.getBounds().getTranslated(d.width, d.height);
		}

		boolean doit = true;

		/*
		 * Prfung ob man in Y-Richtung aus dem Bereich der Parentfigure entfliehen
		 * kann.
		 */
		Rectangle alt = null;
		if (!isAllowMoveOutOfParentsYBounds())
		{

			IFigure parent = f.getParent();
			if (parent != null)
			{
				Rectangle boundsParent = parent.getBounds();
				/*
				 * Nach oben ausbrechen?
				 */
				if (translated.y < boundsParent.y)
				{
					// doit = false;

					/*
					 * Alternative Koordinaten berechnen
					 */
					alt = new Rectangle(translated);
					alt.y = boundsParent.y;
				}

				if (doit)
				{
					/*
					 * Nach unten ausbrechen?
					 */
					int maxY = boundsParent.y + boundsParent.height
							- f.getBounds().height;
					if (translated.y > maxY)
					{
						// doit = false;

						alt = new Rectangle(translated);
						alt.y = maxY;
					}
				}
			}
		}

		if (doit)
		{
			/*
			 * Kollisionsdetektierung
			 */
			IFigure otherFigure = getOtherFigure();
			ECollisionLocation collisionWithOtherFigure = getCollisionWithOtherFigure();
			if ((otherFigure != null) && (collisionWithOtherFigure != null))
			{

				if (collisionWithOtherFigure == ECollisionLocation.top)
				{
					Point top = translated.getTop();

					if (otherFigure.getBounds().contains(top))
					{
						doit = false;
						alt = null;
					} else
					{
						if (otherFigure.getBounds().getBottom().y >= top.y)
						{
							doit = false;

							/*
							 * Alternative Bounds fr das untere Dreieck berechnen: diese wird
							 * (wenn Maus mehr als 1Px liefert) direkt unterhalb des unteren
							 * Dreiecks positioniert.
							 */
							alt = new Rectangle(f.getBounds());
							alt.y = otherFigure.getBounds().bottom() + 1;
						}
					}
				} else if (collisionWithOtherFigure == ECollisionLocation.bottom)
				{
					Point bottom = translated.getBottom();
					if (otherFigure.getBounds().contains(bottom))
					{
						doit = false;
						alt = null;
					} else
					{
						if (otherFigure.getBounds().getTop().y <= bottom.y)
						{
							doit = false;

							/*
							 * Alternative Bounds fr das obere Dreieck berechnen: dieses wird
							 * (wenn Maus mehr als 1Px liefert) direkt oberhalb des unteren
							 * Dreiecks Positioniert.
							 */
							alt = new Rectangle(f.getBounds());
							alt.y = otherFigure.getBounds().getTop().y - 1
									- f.getBounds().height;
						}
					}
				}
			}
		}

		Rectangle figureBoundsOld = f.getBounds();

		if (doit)
		{
			f.setBounds(translated);

			/*
			 * Listener informieren.
			 */
			fireEventBoundsChange(f.getBounds());
		}

		if (alt != null)
		{

			/*
			 * Alternative Bounds setzen. Dies ist z.B. notwendig wenn auf Grund der
			 * Mausgeschwindigkeit mehr als 1 Pixel als Differenz zur alten Position
			 * bermittelt wurde und sonst nicht das obere Ende des unteren Dreiecks
			 * bzw. das untere Ende des oberen Dreiecks getroffen wird.
			 */
			f.setBounds(alt);

			/*
			 * Listener informieren.
			 */
			fireEventBoundsChange(f.getBounds());
		}
	}
	/**
	 * Prft ob nur in Y-Richtung verschoben werden darf.
	 * 
	 * @return Wenn <code>true</code> darf nur in Y-Richtung verschoben werden.
	 */
	public boolean isMoveOnlyAlongYAxis()
	{
		return _moveOnlyAlongYAxis;
	}

	/**
	 * Legt fest ob nur in Y-Richtung verschoben werden darf.
	 * 
	 * @param moveOnlyAlongYAxis
	 */
	public void setMoveOnlyAlongYAxis(boolean moveOnlyAlongYAxis)
	{
		_moveOnlyAlongYAxis = moveOnlyAlongYAxis;
	}

	public boolean isAllowMoveOutOfParentsYBounds()
	{
		return _allowMoveOutOfParentsYBounds;
	}

	public void setAllowMoveOutOfParentsYBounds(
			boolean allowMoveOutOfParentsYBounds)
	{
		_allowMoveOutOfParentsYBounds = allowMoveOutOfParentsYBounds;
	}

	public void setOtherFigure(IFigure otherFigure,
			ECollisionLocation collisionLocation)
	{
		this._otherFigure = otherFigure;
		this._collisionWithOtherFigure = collisionLocation;
	}

	public IFigure getOtherFigure()
	{
		return _otherFigure;
	}

	public ECollisionLocation getCollisionWithOtherFigure()
	{
		return _collisionWithOtherFigure;
	}

	public void mouseReleased(MouseEvent e)
	{
		// System.out.println("DndHelper.mouseReleased()");
	}

	public void mouseClicked(MouseEvent e)
	{
		// System.out.println("DndHelper.mouseClicked()");
	}

	public void mouseDoubleClicked(MouseEvent e)
	{
		// System.out.println("DndHelper.mouseDoubleClicked()");
	}

	public void mousePressed(MouseEvent e)
	{
		// System.out.println("DndHelper.mousePressed()");
		start = e.getLocation();
		e.consume();
	}

	@Override
	public void mouseEntered(MouseEvent me)
	{
		// System.out.println("DndHelper.mouseEntered()");
	}

	@Override
	public void mouseExited(MouseEvent me)
	{
		// System.out.println("DndHelper.mouseExited()");
	}

	@Override
	public void mouseHover(MouseEvent me)
	{
		// System.out.println("DndHelper.mouseHover()");
	}

	@Override
	public void mouseMoved(MouseEvent me)
	{
		// System.out.println("DndHelper.mouseMoved()");
	}

	public enum ECollisionLocation
	{
		top, bottom;
	}

	/**
	 * Fgt einen Listener hinzu.<br>
	 * Dieser wird mit dem Property {@link DndHelper#PROP_VERSCHOBEN} darber
	 * informiert, wenn eine Verschiebung der Pfeilspitzen statt fand. Im OldValue
	 * wird die bisherige Position als {@link Rectangle} bermittelt, im NewValue
	 * die neue Position als {@link Rectangle}.
	 * 
	 * @param listener
	 *          Der hinzuzufgende Listener.
	 */
	public void addListener(PropertyChangeListener listener)
	{
		_pcs.addPropertyChangeListener(listener);
	}

	/**
	 * Entfernt einen Listener, dieser wird fortan ber keinerlei Changes mehr
	 * informiert.
	 * 
	 * @param listener
	 *          Der zu entfernende Listener.
	 */
	public void removeListener(PropertyChangeListener listener)
	{
		System.out.println("  REMOVED DndHelper.removeListener()");
		_pcs.removePropertyChangeListener(listener);
	}

}