/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.streckenprofil.sts.a8.histdata;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.ui.part.WorkbenchPart;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Activator;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.daten.StreckenProfilUIProvider;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.StreckenprofilEditor;
import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.Dataset;
import de.bsvrz.dav.daf.main.archive.ArchiveData;
import de.bsvrz.dav.daf.main.config.SystemObject;

public class HistorischeDatenJob  extends Job 
{
  public final static String PROP_ZUSTAND = "HistorischeDaten.Zustand"; 
  public final static String ZUSTAND_ARCHIVE_REQUEST = "Archivanfrage";
  public final static String ZUSTAND_DATA_PROCESSING = "Verarbeitung";
  public final static String ZUSTAND_PAUSE_STATE = "Pause";
  public final static String ZUSTAND_IDLE_STATE = "Beendet";
  
  protected final static long TAKT = 100;
  protected final static long IDLE = 500;
  
  private PropertyChangeSupport propertyChangeSupport; 
  
  private Date start;
  private Date ende;
  private IDatenZusammenstellung datenZusammenstellung;
  private SystemObject archiveKonfigObj;
  
  private boolean paused = false;
  
  private double simulationsGeschwindigkeit = 1.0;
  
  /**
   * Seit dem letzten Start bereits abgearbeitete Zeit in ms
   */
  private long elapsedTime;
  
  /**
   * Simulationszeit beim letzten Start
   */
  private long simuBaseTime;
  
  /**
   * Der GUI-Kontext, in dem der Job luft
   */
  private WorkbenchPart context;
  
  
//  protected final static SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("dd.MM.yy HH:mm:ss:SSS");
  protected final static SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("dd.MM.yy HH:mm:ss");
  
  public HistorischeDatenJob(Date start, Date ende, IDatenZusammenstellung datenZusammenstellung, 
      WorkbenchPart context, SystemObject archiveKonfigObj)
  {
    super("Historische Daten");
    
    propertyChangeSupport = new PropertyChangeSupport(this);
    
    Assert.isNotNull(start, "Startzeit darf nicht null sein");
    Assert.isNotNull(ende, "Endezeit darf nicht null sein");
    Assert.isTrue(start.before(ende), "Startzeit muss kleiner als Endezeit sein");
    Assert.isNotNull(datenZusammenstellung, "Datenzusammenstellung darf nicht null sein");
    Assert.isTrue(datenZusammenstellung.getDaten() != null && !datenZusammenstellung.getDaten().isEmpty(),
        "Es mssen Daten in der Datenzusammenstellung vorhanden sein");
        
    this.start = start;
    this.ende = ende;
    this.datenZusammenstellung = datenZusammenstellung;
    this.context = context;
    this.archiveKonfigObj = archiveKonfigObj;
  }

  @Override
  protected IStatus run(IProgressMonitor monitor)
  {
    IStatus status = Status.OK_STATUS;
    IArchivAnfrage archivAnfrage = null;
    
    monitor.beginTask("Simulation mit historischen Daten", 1000 /*IProgressMonitor.UNKNOWN*/);
    try {
      monitor.subTask("Archivanfrage fr historische Daten");
      
      propertyChangeSupport.firePropertyChange(PROP_ZUSTAND, null, ZUSTAND_ARCHIVE_REQUEST);
      
      archivAnfrage = new ArchivAnfrage(start, ende, datenZusammenstellung, 
          archiveKonfigObj);
      ArchivDatenPuffer archivDatenPuffer = new ArchivDatenPuffer(archivAnfrage);
      
      int workDone = 10;
      monitor.worked(workDone);
      
      monitor.subTask("Verarbeite historische Daten");
      
      propertyChangeSupport.firePropertyChange(PROP_ZUSTAND, ZUSTAND_ARCHIVE_REQUEST, ZUSTAND_DATA_PROCESSING);
      
      elapsedTime = 0;
      simuBaseTime = start.getTime();
      long startTime = System.currentTimeMillis();
      
      String simulationsZeit = DATE_FORMAT.format(new Date(simuBaseTime)); 
      
      while (true) {
        
        if (monitor.isCanceled()) {
          break;
        }
        
        Thread.sleep(TAKT);
        
        if (isPaused()) {
          startTime = System.currentTimeMillis();
          continue;
        }
        
        elapsedTime = System.currentTimeMillis() - startTime;
        
        long simuTime = simuBaseTime + Math.round(simulationsGeschwindigkeit * elapsedTime);
       
        int wd = (int)Math.round(((simuTime - start.getTime()) * 1000) / (ende.getTime() - start.getTime()));
        
        if (wd > workDone) {
          monitor.worked(wd - workDone);
          workDone = wd;
//          System.out.println("workDone: " + wd);
        }
        
        String sz = DATE_FORMAT.format(new Date(simuTime));
        if (!sz.equals(simulationsZeit)) {
          simulationsZeit = sz;
//          System.out.println("Simulierte Zeit: " + sz);
//          this.setName("Historische Daten " + simulationsZeit);
        }
        
        int index = archivDatenPuffer.getIndexMinimalerArchivZeitstempel();
        
        List<Dataset> archiveList = null;
        while (index >= 0) {
          ArchiveData archiveData = archivDatenPuffer.get(index);
          long timeStamp = archiveData.getArchiveTime(); 
          if (timeStamp > simuTime)
          {
            break;
          }
//          if (archiveData.getDataType().equals(DataState.DATA)
//              && timeStamp >= start.getTime() && timeStamp <= ende.getTime())
          if (archiveData.getDataType().equals(DataState.DATA) ||
              archiveData.getDataType().equals(DataState.NO_DATA) ||
              archiveData.getDataType().equals(DataState.POSSIBLE_GAP))
          {
//            System.out.println(DATE_FORMAT.format(new Date(archiveData.getArchiveTime())) + "\t" + archiveData.getData());
//            StreckenProfilUIProvider.getInstanz().notifyStreckenProfilUIProviderUpdate(archiveData);
            if (archiveList == null) {
              archiveList = new ArrayList<Dataset>();
            }
            
            
            if (archiveData.getDataDescription().getAttributeGroup().getPid().equals("atg.anzeigeEigenschaftIst")){
              
              String s = DATE_FORMAT.format(new Date(archiveData.getArchiveTime()));
              
//              System.out.println(s + " ---> " + archiveData.getObject() + " : " + archiveData.getData());
            }
            
            archiveList.add(archiveData);
          }else{
//            System.err.println("---> else ArchiveDataTyp: " + archiveData.getDataType());
          }
//          System.out.println("VorTake: " + archiveData + " : " + archivDatenPuffer.getLaenge());
          
          archivDatenPuffer.take(index);
//          System.out.println("NachTake: " + archivDatenPuffer.getLaenge());
          index = archivDatenPuffer.getIndexMinimalerArchivZeitstempel();
        }
        
        if (archiveList != null && !archiveList.isEmpty()) {
          StreckenProfilUIProvider.getInstanz().notifyStreckenProfilUIProviderUpdate(context, archiveList);          
        }
        
        if (simuTime > ende.getTime())
          break;
      }
      if (archivAnfrage != null) {
        archivAnfrage.close();
//        System.out.println("Archivanfrage erfolgreich geschlossen");
        archivAnfrage = null;
      }
       
//      if (!monitor.isCanceled()) {
//        monitor.subTask("Verarbeitung historischer Daten beendet");
//        propertyChangeSupport.firePropertyChange(PROP_ZUSTAND, ZUSTAND_DATA_PROCESSING, ZUSTAND_IDLE_STATE);
//      
//        while (!monitor.isCanceled()) {
//          Thread.sleep(IDLE);
//        }
//      }
    }
    catch (IOException e) {
      String msg = "Verarbeitung historischer Daten ist fehlgeschlagen";
//      Activator.getDefault().getLogger().warning(msg, e);
      status = new Status(IStatus.ERROR, Activator.PLUGIN_ID, msg, e);
//      e.printStackTrace();
    }
    catch (InterruptedException e) {
      String msg = "Verarbeitung historischer Daten ist fehlgeschlagen";
//      Activator.getDefault().getLogger().warning(msg, e);
      status = new Status(IStatus.ERROR, Activator.PLUGIN_ID, msg, e);
//      e.printStackTrace();
    }
    catch (RuntimeException e) {
      String msg = "Verarbeitung historischer Daten ist fehlgeschlagen";
//      Activator.getDefault().getLogger().warning(msg, e);
      status = new Status(IStatus.ERROR, Activator.PLUGIN_ID, msg, e);
//      e.printStackTrace();
    }
    finally {
      if (archivAnfrage != null) {
        archivAnfrage.close();
      }
      monitor.done();
    }
    return status;
  }
  
  public synchronized void addPropertyChangeListener(PropertyChangeListener listener) {
    propertyChangeSupport.addPropertyChangeListener(listener);
  }
  
  public synchronized void removePropertyChangeListener(PropertyChangeListener listener) {
    propertyChangeSupport.removePropertyChangeListener(listener);
  }
  
  
  public synchronized boolean isPaused()
  {
    return paused;
  }

  public synchronized void setPaused(boolean paused)
  {
    if (paused) {
      if (!this.paused) {
        simuBaseTime += Math.round(simulationsGeschwindigkeit * elapsedTime);
        elapsedTime = 0;
        this.paused = true;
        propertyChangeSupport.firePropertyChange(PROP_ZUSTAND, ZUSTAND_DATA_PROCESSING, ZUSTAND_PAUSE_STATE);
      }
    }
    else {
      if (this.paused) {
        this.paused = false;  
        propertyChangeSupport.firePropertyChange(PROP_ZUSTAND, ZUSTAND_PAUSE_STATE, ZUSTAND_DATA_PROCESSING);
      }
    }
  }
  
  /**
   * Liefert die Simulationsgeschwindigkeit.
   * 
   * @return the simulationsGeschwindigkeit
   */
  public final synchronized double getSimulationsGeschwindigkeit() {
    return simulationsGeschwindigkeit;
  }

  /**
   * Setzt die Simulationsgeschwindigkeit.
   * 
   * @param simulationsGeschwindigkeit
   *            the simulationsGeschwindigkeit to set
   */
  public final synchronized void setSimulationsGeschwindigkeit(
      final double simulationsGeschwindigkeit) {

    this.simulationsGeschwindigkeit = simulationsGeschwindigkeit;
  }

}
