/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.tools;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.losb.exceptions.LoggerException;
import de.bsvrz.sys.funclib.losb.exceptions.SenderException;
import de.bsvrz.sys.funclib.losb.kernsoftware.ConnectionManager;
import de.bsvrz.sys.funclib.losb.messages.ErrorMessage;
import de.bsvrz.sys.funclib.losb.util.Util;

/**
 * Klasse die das Senden immer unter Bercksichtigung der 
 * Sendesteuerung durchfhrt. Nach dem Senden der Daten meldet sich der Sender
 * sofort beim Datenverteiler ab.
 * Falls keine Exception geworfen wird, und {@link #sent} dennoch false ist, dann
 * gibt es keinen Abnehmer fr die Daten.
 * <br>
 * <b>WICHTIG</b><br>
 * Wenn versucht wird, etwas ohne angemeldeten Empfnger Quelle zu senden, dann wird (korrekterweise)
 * ein Fehler gemeldet. 
 * Problematisch wird es, wenn sofort danach eine Quellen-/Empfngeranmeldung durchgefhrt und das Senden
 * wiederholt wird. <b>Dies fhrt zu einem Fehler<b>, da die Sendesteuerung das Senden immer noch 
 * nicht gestattet. Daher sollte eine Pause zwischen zwei Sendeversuchen liegen. (Siehe JUnit-Tests).
 * 
 * @author beck et al. projects GmbH
 * @author Martin Hilgers
 * @version $Revision: 1.2 $ / $Date: 2012/12/21 09:56:19 $ / ($Author: Pittner $)
 */

public class SimpleSenderMitWartenAufParametrierung implements ClientSenderInterface
{
	/**
	 * Datensatz der zu versendenen Daten.
	 */
	private Data data;
	/**
	 * Objekt, das den Empfnger realisiert.
	 */
	private SystemObject receiver;
	/**
	 * Verbindung zum Datenverteiler.
	 */
	private ClientDavInterface dav;
	/**
	 * Attributgruppen/Aspekt-Kombination
	 */
	private DataDescription dataDescription;
	/**
   * DebugLogger fr Debug-Ausgaben.
	 */
	private static final Debug debug = Debug.getLogger();
	
	/**
	 * Wird als Synchronisationspunkt verwendet. Wird von Sender
	 * mit sent.notifyAll() geweckt, nachdem die Daten gesendet wurden.
	 */
	public Object lock;
	/**
	 * Wird nach dem Senden gesetzt. True, falls das Senden erfolgreich durchgefhrt wurde. False sonst.
	 * Falls keine Exception geworfen wird, und sent dennoch false ist, dann gibt es keinen Abnehmer fr die Daten.
	 */
	public boolean sent;
	/**
	 * Mit null oder einer Fehlermeldung belegt.
	 */
	public String errorMsg;
	/**
	 * Zeigt ob die Methode {@link #dataRequest(SystemObject, DataDescription, byte)}
	 * bereits aufgerufen wurde.
	 */
	private boolean dataRequestCalled = false;

	/**
	 * Erzeugt einen Sender und sendet die Daten als einfacher Sender.
	 * Die Methode wartet solange, bis die Daten gesendet wurden, oder es zu einem Fehler kam.
	 * @param dav		Verbindung zum Datenverteiler
	 * @param receiver Empfnger der Daten
	 * @param dataDescription DataDescription der zu versendenden Daten
	 * @param data Daten, die versendet werden sollen
	 * @param timeout	Maximale Wartezeit in ms. Bei <code>0</code> unbegrenzt.
	 * @return <code>true</code> falls das Senden erfolgreich durchgefhrt wurde.
	 * @throws SenderException Fehler beim Senden.
	 * @throws RuntimeException Kann bei Fehler geworfen werden.
	 */
	public static boolean sendWait(ClientDavInterface dav, SystemObject receiver, DataDescription dataDescription, Data data, long timeout) throws SenderException
	{
		return send(dav, receiver, dataDescription, data, false, timeout);
	}

	/**
	 * Erzeugt einen Sender und sendet die Daten als Quelle.
	 * @param dav		Verbindung zum Datenverteiler
	 * @param receiver Empfnger der Daten
	 * @param dataDescription DataDescription der zu versendenden Daten
	 * @param data Daten, die versendet werden sollen
	 * @return Quelle
	 * @throws SenderException Fehler beim Senden.
	 */
	public static SimpleSenderMitWartenAufParametrierung source(ClientDavInterface dav, SystemObject receiver, DataDescription dataDescription, Data data) throws SenderException
	{
		SimpleSenderMitWartenAufParametrierung sender = new SimpleSenderMitWartenAufParametrierung(dav, receiver, dataDescription, data, true);
		return sender;
	}
	
	/**
	 * Erzeugt einen Sender und sendet die Daten als einfache Quelle.
	 * Die Methode wartet solange, bis die Daten gesendet wurden, oder es zu einem Fehler kam.
	 * @param dav		Verbindung zum Datenverteiler
	 * @param receiver Empfnger der Daten
	 * @param dataDescription DataDescription der zu versendenden Daten
	 * @param data Daten, die versendet werden sollen
	 * @param timeout	Maximale Wartezeit in ms. Bei <code>0</code> unbegrenzt.
	 * @return False, falls die Daten wegen negativer Sendesteuerung nicht gesendet wurden.
	 * @throws SenderException Falls es beim Senden zu einem Fehler kommt.
	 * @throws RuntimeException Kann bei Fehler geworfen werden.
	 */
	public static boolean  sourceWait(ClientDavInterface dav, SystemObject receiver, DataDescription dataDescription, Data data, long timeout) throws SenderException
	{
		return send(dav, receiver, dataDescription, data, true, timeout);
	}
	
	/**
	 * Initialisiert das Lock-Objekt.
	 */
	private SimpleSenderMitWartenAufParametrierung()
	{
		lock = new Object();
	}
	
	/**
	 * Erzeugt den Sender und sendet die Daten.
	 * @param dav		Verbindung zum Datenverteiler
	 * @param receiver	Empfnger der Daten
	 * @param dataDescription DataDescription der zu versendenden Daten
	 * @param data		Daten, die versendet werden sollen
	 * @param source	True wenn die Daten als Quelle versendet werden sollen. Ist source == false meldet sich {@link SimpleSenderMitWartenAufParametrierung} als einfacher Sender an.
	 * @throws SenderException Falls es bei der Sendeanmeldung zu einem Fehler kommt.
	 */
	public SimpleSenderMitWartenAufParametrierung(ClientDavInterface dav, SystemObject receiver, DataDescription dataDescription, Data data, boolean source) throws SenderException
	{
		this();
		init(dav, receiver, dataDescription, data, source);
	}
	
	/**
	 * Objektinitialisierung.
	 * @param dav		Verbindung zum Datenverteiler
	 * @param receiver	Empfnger der Daten
	 * @param dataDescription DataDescription der zu versendenden Daten
	 * @param data		Daten, die versendet werden sollen
	 * @param source	True wenn die Daten als Quelle versendet werden sollen. Ist source == false meldet sich {@link SimpleSenderMitWartenAufParametrierung} als einfacher Sender an.
	 * @throws SenderException Falls es bei der Sendeanmeldung zu einem Fehler kommt.
	 */
	private void init(ClientDavInterface dav, SystemObject receiver, DataDescription dataDescription, Data data, boolean source) throws SenderException
	{
		this.dav = dav;
		this.data = data;
		this.receiver = receiver;
		this.dataDescription = dataDescription;
		
		sent = false;
		errorMsg = null;
		
		SenderRole role = (source) ? SenderRole.source() : SenderRole.sender();
		try
		{
			ConnectionManager.subscribeSender(dav, this, receiver, dataDescription, role);
		}
		catch (OneSubscriptionPerSendData e)	//Es existiert bereits eine Anmeldung. Strt nicht weiter.
		{
			String msg = receiver.getNameOrPidOrId() + ":"+ dataDescription;
			debug.warning(ErrorMessage.MULIPLE_SUBSCRIPTIONS+msg);
		}
		catch (RuntimeException re)
		{
			throw new SenderException(ErrorMessage.CAN_NOT_SUBSCRIBE + Util.getStackTrace(re), LoggerException.WARNING);
		}
	}
	
	/**
	 * Erzeugt einen Sender und sendet die Daten.
	 * Die Methode wartet solange, bis die Daten gesendet wurden, oder es zu einem Fehler kam.
	 * Kann <code>RuntimException</code> werfen!
	 * @param dav		Verbindung zum Datenverteiler
	 * @param receiver Empfnger der Daten
	 * @param dataDescription DataDescription der zu versendenden Daten
	 * @param data Daten, die versendet werden sollen
	 * @param source	True wenn als Quelle gesendet werden soll. False, wenn als einfacher Sender gesendet werden soll.
	 * @param timeout	Maximale Wartezeit in ms. Bei <code>0</code> unbegrenzt.
	 * @return <code>true</code> falls das Senden erfolgreich durchgefhrt wurde.
	 * @throws SenderException Falls es whrend des Sendens zu einem Fehler kam.
	 * @throws RuntimeException Kann bei Fehler geworfen werden.
	 */
	private static boolean send(ClientDavInterface dav, SystemObject receiver, DataDescription dataDescription, Data data, boolean source, long timeout) throws SenderException
	{
		SimpleSenderMitWartenAufParametrierung sender = new SimpleSenderMitWartenAufParametrierung();
		try
		{
			sender.init(dav, receiver, dataDescription, data, source);

			synchronized (sender.lock)
			{
				while (!sender.dataRequestCalled)
					sender.lock.wait(timeout);		//warten, bis gesendet oder timeout
			}
			
			if (!sender.sent) // wenn nicht gesendet, dann Sender abmelden
			{
				ConnectionManager.unsubscribeSender(dav, sender, receiver, dataDescription);
			}
			
			return sender.sent;
		}
		catch (InterruptedException e)
		{
			sender.errorMsg = Util.getStackTrace(e);
			return false;
		}
		catch (SenderException e)
		{
			sender.errorMsg = e.getMessage();
			return false;
		}
		finally
		{
			//ggf. Fehlermeldung ausgeben (auch nicht-kritische)
			if (sender.errorMsg != null)
			{
				debug.fine(sender.errorMsg);
				//berprfen, ob das Senden funktioniert hat
				if (!sender.sent)
					throw new SenderException(sender.errorMsg, LoggerException.WARNING);
			}
		}
	}

	/**
	 * Sendesteuerung. Callback Methode fr den Datenverteiler.
	 * Wird der Versand von Daten erlaubt, so werden die Daten sofort abgesetzt. Danach
	 * meldet sich der Sender unverzglich ab.
	 * Wird der Versand der Daten nicht erlaubt, so werden sie verworfen.
	 * @param object Das in der zugehrigen Sendeanmeldung angegebene Objekt, auf das sich die Sendesteuerung bezieht.
	 * @param dataDescription Beschreibende Informationen zu den angemeldeten Daten auf die sich die Sendesteuerung bezieht.
	 * @param state Status der Sendesteuerung. Kann einen der Werte START_SENDING, STOP_SENDING, STOP_SENDING_NO_RIGHTS, STOP_SENDING_NOT_A_VALID_SUBSCRIPTION enthalten.
	 */
	public void dataRequest(SystemObject object, DataDescription dataDescription, byte state)
	{
		try
		{
			sent = false;
			switch (state)
			{		
			case ClientSenderInterface.START_SENDING:
				ResultData rd = new ResultData(receiver, this.dataDescription, System.currentTimeMillis(), data);
				try
				{
					dav.sendData(rd);
					sent = true;
				}
				catch (SendSubscriptionNotConfirmed e)
				{
					//sollte nie aufgerufen werden, da ja gerade berprft wurde, ob Sender senden darf
					errorMsg = ErrorMessage.SENDING_NOT_ALLOWED+e.getMessage();
				}
				catch (RuntimeException e)
				{
					errorMsg = ErrorMessage.CAN_NOT_SEND + Util.getStackTrace(e);	
				}
				break;
			case ClientSenderInterface.STOP_SENDING:
				sent = false;
				break;
			default:
				errorMsg = ErrorMessage.SENDING_NOT_ALLOWED +object.getNameOrPidOrId()+":"+dataDescription + ", Sendsteuerung: " + state;
				sent = false;
			}
		}
		finally
		{
			//	Sender abmelden, wenn erfolgreich versendet

			if (sent)
			{
				ConnectionManager.unsubscribeSender(dav, this, object, dataDescription);

				synchronized (lock)
				{
					dataRequestCalled  = true;
					lock.notifyAll();
				}
			}
		}
	}

	/**
	 * Sendesteuerung ist erwnscht. Wird vom Datenverteiler aufgerufen.
	 * Liefert deshalb immer true zurck.
	 * @param object Das in der zugehrigen Sendeanmeldung angegebene System-Objekt.
	 * @param dataDescription Die in der zugehrigen Sendeanmeldung angegebenen beschreibenden Informationen der angemeldeten Daten.
	 * @return <code>true</code>, falls Sendesteuerungen gewnscht sind, sonst <code>false</code>.
	 * @see ClientSenderInterface#isRequestSupported(SystemObject, DataDescription)
	 */
	public boolean isRequestSupported(SystemObject object, DataDescription dataDescription)
	{
		return true;
	}
}
