/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.entfernungsangaben;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;
import java.util.Vector;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.DaVKommunikationSBA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;

/**
 * Klasse speichert die Parameter fr die Entfernungsangaben
 * @version $Revision: 1.1 $ / $Date: 2013/06/05 14:17:02 $ / ($Author: ObertM $)
 * @author Dambach-Werke GmbH
 * @author Matthias Obert
 */
public class VerwaltungParaEntfernungsAngaben
{
	/**
	 * Einzige Instanz der Klasse
	 */
  private static VerwaltungParaEntfernungsAngaben _instance;

  /**
   * Pid der Attributgruppe mit den Parametern
   */
  private String _pidAtg = "atg.sswSbaEntfernungsAngaben";
  
  /**
   * Hashmap mit den Treppenfunktionen, als Key dient die Bezeichnung der Treppenfunktion, als Value
   * die Treppenfunktion selbst
   */
  private HashMap<String, TreppenFunktion> m_treppenFunktionen = new HashMap<String, TreppenFunktion>();
  
  /**
   * Pid der SBA
   */
  private String _pidSBA;

  /**
   * Privater Konstruktor der Klasse
   */
  private VerwaltungParaEntfernungsAngaben ()
  {
  	_pidSBA = Konfig.getInstance().getPidSBA();
    
    /*
     * Daten lesen ber DaVKommunikation
     */
  	final DaVKommunikationSBA kom = DaVKommunikationSBA.getInstanz();
  	kom.addDatenArt(_pidSBA, _pidAtg, "asp.parameterSoll", ReceiverRole.receiver());
  	
  	kom.addPropertyChangeListener(new PropertyChangeListener()
		{
			@Override
			public void propertyChange(PropertyChangeEvent evt)
			{
				if (evt.getPropertyName().equals(_pidAtg))
				{
					ResultData resultData = kom.getData(_pidAtg);
					if (resultData != null)
						auswerteAttributgruppe(resultData);
				}
			}
		});

		ResultData resultData = kom.getData(_pidAtg);
		if (resultData != null)
			auswerteAttributgruppe(resultData);
  }
  
  /**
   * Methode liefert die Singleton Instanz der Klasse
   * @return Singleton Instanz der Klasse
   */
  public static VerwaltungParaEntfernungsAngaben getInstance ()
  {
    if (_instance == null)
    {
      synchronized (VerwaltungParaEntfernungsAngaben.class)
      {
        if (_instance == null)
        {
          _instance = new VerwaltungParaEntfernungsAngaben ();
        }
      }
    }
    
    return _instance;
  }
  
  public void auswerteAttributgruppen (ResultData[] results)
  {
    int anzahl = results.length;

    // Daten aus Eingangsqueue bearbeiten      
    
    for (int i = 0; i < anzahl; i++)
    {
      ResultData dat = results[i];
  
      if (dat.getDataDescription().getAttributeGroup().getPid().equals(_pidAtg))
      {
        auswerteAttributgruppe (dat);
        
//        System.out.println( "EntfernungsAngaben.auswerteAttributgruppen()\n" + toString() );
      }
    }
  }
  
  /**
   * Methode zum Auswerten der Attributgruppe 'atg.sswSbaTrichterung'
   * @param dat empfangender Datensatz
   */
  private void auswerteAttributgruppe( ResultData dat )
  {
    synchronized (m_treppenFunktionen)
    {
      m_treppenFunktionen.clear();

      if (dat.hasData())
      {
        Data data = dat.getData();
          
        Array a = data.getArray( "Treppenfunktionen" );
        int anzahl = a.getLength();
        
        for (int x = 0; x < anzahl; ++x)
        {
          TreppenFunktion tf = new TreppenFunktion ();
          tf.auswerteAttributgruppe (a.getItem( x ));
          
          m_treppenFunktionen.put( tf.getBezeichnung(), tf );
        }
      }
    }
  }

  /**
   * Methode zum Ausgeben der Klasse als String
   */
  @Override
  public String toString ()
  {
    String s = "";
    
    s += "Treppenfunktionen:\n";
    
    for (TreppenFunktion tf : getTreppenFunktionen())
      s += tf.toString();
    
    return s;
  }
  
  /**
   * Methode liefert die Treppenfunktion zu einer bergebenen Bezeichnung
   * @param bezeichnung Bezeichnung der gesuchten Treppenfunktion
   * @return Treppenfunktion sofern vorhanden, sonst null
   */
  public TreppenFunktion getTreppenFunktion (String bezeichnung)
  {
  	synchronized (m_treppenFunktionen)
    {
	    if (m_treppenFunktionen.containsKey(bezeichnung))
	    	return m_treppenFunktionen.get(bezeichnung);
    }
    
    return null;  
  }

  /**
   * Methode liefert einen Vektor mit den Treppenfunktionen zuurck
   * @return Vektor mit den Treppenfunktionen
   */
  public Vector<TreppenFunktion> getTreppenFunktionen()
  {
    Vector<TreppenFunktion> v = new Vector<TreppenFunktion>();
    
    synchronized (m_treppenFunktionen)
    {
      for (TreppenFunktion tf : m_treppenFunktionen.values())
        v.add( tf );
    }
    
    return v;
  }

  /**
   * Methode setzt die Variable m_treppenFunktionen
   * @param treppenFunktionen Wert auf den die Variable m_treppenFunktionen gesetzt werden soll
   */
  public void setTreppenFunktionen( Vector<TreppenFunktion> treppenFunktionen )
  {
    synchronized (m_treppenFunktionen)
    {
    	m_treppenFunktionen.clear();
    	for (TreppenFunktion tf : treppenFunktionen)
        m_treppenFunktionen.put(tf.getBezeichnung(), tf);
    }
  }
  
  /**
   * Methode fgt eine neue Treppenfunktion hinzu
   * @param treppenFunktion neue Treppenfunktion
   */
  public void addTreppenFunktion (TreppenFunktion treppenFunktion)
  {
  	synchronized (m_treppenFunktionen)
    {
	    m_treppenFunktionen.put(treppenFunktion.getBezeichnung(), treppenFunktion);
    }
  }

  /**
   * Methode lscht eine Treppenfunktion
   * @param bezeichnung Bezeichnung der Treppenfunktion
   */
  public void removeTreppenFunktion (String bezeichnung)
  {
  	synchronized (m_treppenFunktionen)
    {
  		if (m_treppenFunktionen.containsKey(bezeichnung))
	      m_treppenFunktionen.remove (bezeichnung);
    }
  }

  /**
   * Methode liefert die Parameter als Attributgruppe fr den DaV
   * @return Attributgruppe
   */
  public Data getData ()
  {
  	return erzeugeAttributgruppe();
  }
  
  /**
   * Methode zum Erzeugen einer Attributgruppe mit den Parametern
   * @return Attributgruppe mit den Parametern
   */
  public Data erzeugeAttributgruppe ()
  {
	  ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection == null)
      return null;

    String pidAtg = _pidAtg;
    
    // Attributgruppe zusammensetzen und versenden
      
    AttributeGroup atg = (AttributeGroup) connection.getDataModel().getAttributeGroup(pidAtg);
    Data data = connection.createData(atg );

    synchronized (m_treppenFunktionen)
    {
	    Array a1 = data.getArray("Treppenfunktionen");
	    
	    Vector<TreppenFunktion> treppenFunktionen = getTreppenFunktionen();
	    
	    int anzahl = treppenFunktionen.size();
    
      a1.setLength(anzahl);
      
      for (int i=0; i<anzahl; ++i)
      {
        Data d1 = a1.getItem(i);
        TreppenFunktion tf = treppenFunktionen.get(i);
        
        d1.getTextValue("Bezeichnung").setText(tf.getBezeichnung());
        
        Array a2 = d1.getArray("Stufen");
        int anzahlStufen = tf.getStufen().size();
        a2.setLength(anzahlStufen);
        
        for (int j=0; j<anzahlStufen; ++j)
        {
        	Data d2 = a2.getItem(j);
        	Stufe stufe = tf.getStufen().get(j);
        	
        	d2.getUnscaledValue("UntereEntfernung").set(stufe.getVon());
        	d2.getUnscaledValue("ObereEntfernung").set(stufe.getBis());
        	d2.getReferenceValue("AnzeigeInhalt").setSystemObject(stufe.getWvzInhalt().getAnzeigenInhalt());
        }
      }
    }
    
    return data;
  }
}


