/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.prio;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.DaVKommunikationSBA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

/**
 * Klasse enthlt die Parameter fr die Priorisierung. Die Klasse wird als Singelton
 * ausgelegt
 * @author Dambach Werke GmbH
 * @author Matthias Obert
 *
 */
public class VerwaltungParaPriorisierung
{
	/**
	 * Einzige Instanz der Klasse
	 */
  private static VerwaltungParaPriorisierung _instance;
  
  /**
   * Defaultprioritt fr alle Inhalte
   */
  private int m_defaultPrioritaet = 0;
  
  /**
   * Parameter die fr die gesamte Anlage gelten
   */
  private RegelnProAnzeigeTyp m_parameterAnlage = new RegelnProAnzeigeTyp();

  /**
   * Parametersatz pro Anzeigequerschnitt
   */
  private HashMap<SystemObject, RegelnProAnzeigeTyp> m_parameterProAnzeigeQuerschnitt = new HashMap<SystemObject, RegelnProAnzeigeTyp>();
  
  /**
   * Parametersatz pro Anzeige
   */
  private HashMap<SystemObject, RegelnProAnzeigeTyp> m_parameterProAnzeige = new HashMap<SystemObject, RegelnProAnzeigeTyp>();

  /**
   * Verwaltung der Listener
   */
  private PropertyChangeSupport m_pcs = new PropertyChangeSupport (this);

  /**
   * Anlage fr die die SWE arbeitet
   */
  private String m_anlage;
  
  /**
   * Mutex zum synchronisierten Zugriff auf die Parameter
   */
  private Object m_mutex = new Object ();

  /**
   * Pid der SBA
   */
  private String _pidSBA;

  /**
   * Pid der Attributgruppe
   */
  private String _pidAtg = "atg.sswSbaPriorisierung";

  /**
   * Privater Konstruktor der Klasse
   */
  private VerwaltungParaPriorisierung ()
  {
  	_pidSBA = Konfig.getInstance().getPidSBA();

    /*
     * Daten lesen ber DaVKommunikation
     */
  	final DaVKommunikationSBA kom = DaVKommunikationSBA.getInstanz();
  	kom.addDatenArt(_pidSBA, _pidAtg, "asp.parameterSoll", ReceiverRole.receiver());
  	
  	kom.addPropertyChangeListener(new PropertyChangeListener()
		{
			@Override
			public void propertyChange(PropertyChangeEvent evt)
			{
				if (evt.getPropertyName().equals(_pidAtg))
				{
					ResultData resultData = kom.getData(_pidAtg);
					if (resultData != null)
						auswerteAttributgruppe(resultData);
				}
			}
		});

		ResultData resultData = kom.getData(_pidAtg);
		if (resultData != null)
			auswerteAttributgruppe(resultData);
  }

  /**
   * Methode liefert die Singleton Instanz der Klasse
   * @return Singleton Instanz der Klasse
   */
  public static VerwaltungParaPriorisierung getInstance ()
  {
    if (_instance == null)
    {
      synchronized (VerwaltungParaPriorisierung.class)
      {
        if (_instance == null)
        {
          _instance = new VerwaltungParaPriorisierung ();
        }
      }
    }
    
    return _instance;
  }
  
  /**
   * Methode fgt einen Listener der Klasse hinzu
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void addPropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.addPropertyChangeListener( pcl );    
  }
  
  /**
   * Methode entfernt einen Listener aus der Klasse
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void removePropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.removePropertyChangeListener( pcl );    
  }

  /**
   * Methode benachrichtigt die Listener
   */
  private void benachrichtigeListener ()
  {
    m_pcs.firePropertyChange( "Aktualisierung", null, null );
  }

  /**
   * Methode zum Ausgeben der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    synchronized (m_mutex)
    {
      s += "Defaultprioritt: " + m_defaultPrioritaet + "\n";
      s += m_parameterAnlage.toString() + "\n";    
      s += m_parameterProAnzeigeQuerschnitt.toString() + "\n";
      s += m_parameterProAnzeige.toString() + "\n";
    }
    
    return s;
  }

  /**
   * Methode zum Auswerten der empfangenen Attributgruppen
   * @param results empfangenen Attributgruppen
   */
  public void auswerteAttributgruppen (ResultData[] results)
  {
    int anzahl = results.length;

    // Daten aus Eingangsqueue bearbeiten      
    
    for (int i = 0; i < anzahl; i++)
    {
      ResultData dat = results[i];
  
      
      if (dat.getDataDescription().getAttributeGroup().getPid().equals("atg.sswSbaPriorisierung"))
      {
        auswerteAttributgruppe (dat);
        
        benachrichtigeListener();
        
//        System.out.println(toString());
      }
    }
  }
  
  /**
   * Methode zum Auswerten der Attributgruppe 'atg.sswSbaPriorisierung'
   * @param dat empfangender Datensatz
   */
  public void auswerteAttributgruppe( ResultData dat )
  {
    synchronized (m_mutex)
    {
      m_defaultPrioritaet = 0;
      m_parameterAnlage.clear();
      m_parameterProAnzeigeQuerschnitt.clear();
      m_parameterProAnzeige.clear();
      
      if (dat.hasData())
      {
        Data data = dat.getData();
        
        // Defaulprioritt der Anlage
        
        m_defaultPrioritaet = data.getItem( "Defaultprioritt" ).asScaledValue().intValue();
        
        Array a = data.getArray( "AnlageRegeln" );
        auswerteAttributgruppeAnlage( a );
        
        a = data.getArray( "AnzeigeQuerschnitt" );
        auswerteAttributgruppeQuerschnitt (a);
        
        a = data.getArray( "Anzeige");
        auswerteAttributgruppeAnzeige (a);
      }
    }
    
//    System.out
//        .println("VerwaltungParaPriorisierung.auswerteAttributgruppeAnlage()");
//    System.out.println(m_parameterAnlage.toString());
  }

  /**
   * Methode zum Auswerten der fr die gesamte Anlage geltenden Parameter
   * @param array Array mit Parametern
   */
  private void auswerteAttributgruppeAnlage (Array array)
  {
    m_parameterAnlage.auswerteAttributgruppe( array );
  }

  /**
   * Methode zum Auswerten der Anzeigequerschnitt spezifischen Parameter
   * @param array Array mit Parametern fr die Anzeigequerschnitte
   */
  private void auswerteAttributgruppeQuerschnitt( Array array )
  {
    int anzahl = array.getLength(); 
    
    synchronized (m_parameterProAnzeigeQuerschnitt)
    {
      m_parameterProAnzeigeQuerschnitt.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        Data d1 = array.getItem( x );
        
        SystemObject anzeigeQuerschnitt = d1.getReferenceValue( "AnzeigeQuerschnitt" ).getSystemObject();
        Array a = d1.getArray( "Parameter" );
        
        RegelnProAnzeigeTyp regeln  = new RegelnProAnzeigeTyp ();
        regeln.auswerteAttributgruppe( a );
        
        m_parameterProAnzeigeQuerschnitt.put (anzeigeQuerschnitt, regeln);
      }
    }
  }

  /**
   * Methode zum Auswerten der Anzeige spezifischen Parameter
   * @param array Array mit Parametern fr die Anzeige
   */
  private void auswerteAttributgruppeAnzeige (Array array )
  {
    int anzahl = array.getLength(); 
    
    synchronized (m_parameterProAnzeige)
    {
      m_parameterProAnzeige.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        Data d1 = array.getItem( x );
        
        SystemObject anzeige = d1.getReferenceValue( "Anzeige" ).getSystemObject();
        Array a = d1.getArray( "Parameter" );
        
        RegelnProAnzeigeTyp regeln  = new RegelnProAnzeigeTyp ();
        regeln.auswerteAttributgruppe( a );
        
        m_parameterProAnzeige.put (anzeige, regeln);
      }
    }
  }

  /**
   * Methode fgt eine Regel fr die allgemeinen Parameter der Anlage hinzu
   * @param wvzTyp WVZ-Typ (Anzeigetyp)
   * @param inhalt WVZ-Inhalt
   * @param prio Zugeordnete Prioritt
   * @param schaltgrund Schaltgrund (Aspekt) fr den die Prio gelten soll, fr die Defaultprioritt muss der Schaltgrund null sein
   */
  public void addRegelAllgemein( SystemObject wvzTyp, WVZInhalt inhalt, int prio, Aspect schaltgrund )
  {
    synchronized (m_mutex)
    {
      ListeMitRegeln parameter = m_parameterAnlage.getParameter( wvzTyp ); 
      parameter.addRegel( inhalt, prio, schaltgrund );
    }
  }
  

  /**
   * Methode fgt eine Regel fr die Anzeigenspezifischen Parameter hinzu
   * @param anzeige Anzeige fr die die Regel gelten soll
   * @param inhalt WVZ-Inhalt
   * @param prio Zugeordnete Prioritt
   * @param schaltgrund Schaltgrund (Aspekt) fr den die Prio gelten soll, fr die Defaultprioritt muss der Schaltgrund null sein
   */
  public void addRegelAnzeige( SystemObject anzeige, WVZInhalt inhalt, int prio, Aspect schaltgrund )
  {
  	SystemObject wvzTyp = Konfig.getInstance().getWzgTypVonAnzeige(anzeige.getPid());
    
    if (!m_parameterProAnzeige.containsKey( anzeige ))
    {
      RegelnProAnzeigeTyp rpat = new RegelnProAnzeigeTyp ();
      m_parameterProAnzeige.put( anzeige, rpat );
    }
    
    RegelnProAnzeigeTyp rpat = m_parameterProAnzeige.get( anzeige );
    ListeMitRegeln parameter = rpat.getParameter( wvzTyp );
    parameter.addRegel( inhalt, prio, schaltgrund );
  }
  
  /**
   * Methode liefert die Variable m_anlage zurck
   * @return Variable m_anlage
   */
  public String getAnlage()
  {
    return m_anlage;
  }

  /**
   * Methode setzt die Variable m_anlage
   * @param anlage Wert auf den die Variable m_anlage gesetzt werden soll
   */
  public void setAnlage( String anlage )
  {
    m_anlage = anlage;
  }

  /**
   * Methode erzeugt eine Attributgruppe mit den aktuellen Parametern fr den daV
   */
  public Data erzeugeAttributgruppe()
  {
	  ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection == null)
      return null;

    // Attributgruppe zusammensetzen und versenden
    
    AttributeGroup atg = (AttributeGroup) connection.getDataModel().getAttributeGroup(_pidAtg);
    Data data = connection.createData(atg );

    data.getUnscaledValue ( "Defaultprioritt" ).set( m_defaultPrioritaet );
  
    Array a1 = data.getArray( "AnlageRegeln" );
    speichereParaAnlage (connection, a1);
    
    Array a2 = data.getArray( "AnzeigeQuerschnitt" );
    speichereParaAnzeigeQuerschnitt (connection, a2);
    
    Array a3 = data.getArray( "Anzeige" );
    speichereParaAnzeige (connection, a3);

    return data;
  }

  /**
   * Methode speichert die Anlagen spezifischen Parameter
   * @param connection 
   * @param array Array fr die Anlagen spezifischen Parameter
   */
  private void speichereParaAnlage( ClientDavInterface connection, Array array )
  {
    speichereParaProAnzeigeTyp (connection, array, m_parameterAnlage);
    
  }

  /**
   * Methode speichert die Anzeigequerschnitt spezifischen Parameter
   * @param array Array fr die Anzeigequerschnitt spezifischen Parameter
   */
  private void speichereParaAnzeigeQuerschnitt( ClientDavInterface connection, Array array )
  {
    HashMap<SystemObject, RegelnProAnzeigeTyp> paraAq = m_parameterProAnzeigeQuerschnitt;

    int anzahlAQ = 0;
    
    Iterator<Entry<SystemObject, RegelnProAnzeigeTyp>> it = paraAq.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<SystemObject, RegelnProAnzeigeTyp> entry = it.next();
      
      SystemObject anzeigeQuerschnitt = entry.getKey();
      RegelnProAnzeigeTyp rpat = entry.getValue();
      
      if (rpat.isVersorgt())
      	anzahlAQ++;
    }
    
//    int anzahlAQ = paraAq.size();
    
    array.setLength( anzahlAQ );
    
    int i=0;
    it = paraAq.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<SystemObject, RegelnProAnzeigeTyp> entry = it.next();
      
      SystemObject anzeigeQuerschnitt = entry.getKey();
      RegelnProAnzeigeTyp rpat = entry.getValue();
  
      if (rpat.isVersorgt())
      {
	      Data dataAQ = array.getItem( i++ );
	      
	      dataAQ.getReferenceValue( "AnzeigeQuerschnitt" ).setSystemObject( anzeigeQuerschnitt );
	      Array arrayRegeln = dataAQ.getArray( "Parameter" );
	      
	      speichereParaProAnzeigeTyp(connection, arrayRegeln, rpat);
      }
    }
  }

  /**
   * Methode speichert die Anzeigequerschnitt spezifischen Parameter
   * @param array Array fr die Anzeige spezifischen Parameter
   */
  private void speichereParaAnzeige( ClientDavInterface connection, Array array )
  {
    HashMap<SystemObject, RegelnProAnzeigeTyp> paraAnzeige = m_parameterProAnzeige;
    
    int anzahlAnzeigen = 0;
    Iterator<Entry<SystemObject, RegelnProAnzeigeTyp>> it = paraAnzeige.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<SystemObject, RegelnProAnzeigeTyp> entry = it.next();
      
      SystemObject anzeige = entry.getKey();
      RegelnProAnzeigeTyp rpat = entry.getValue();

      if (rpat.isVersorgt())
      	anzahlAnzeigen++;
    }
    
//    int anzahlAnzeigen = paraAnzeige.size();

    array.setLength( anzahlAnzeigen );
    
    int i=0;
    it = paraAnzeige.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<SystemObject, RegelnProAnzeigeTyp> entry = it.next();
      
      SystemObject anzeige = entry.getKey();
      RegelnProAnzeigeTyp rpat = entry.getValue();

      if (rpat.isVersorgt())
      {
	      Data dataAQ = array.getItem( i++ );
	      
	      dataAQ.getReferenceValue( "Anzeige" ).setSystemObject( anzeige );
	      Array arrayRegeln = dataAQ.getArray( "Parameter" );
	
	      speichereParaProAnzeigeTyp(connection, arrayRegeln, rpat);
      }
    }
  }

  /**
   * Methode speichert die Parameter pro Anzeigetyp
   * @param connection 
   * @param array Array fr die Parameter
   * @param regelnProAnzeigeTyp Regeln unterschieden in Anzeigetypen
   */
  private void speichereParaProAnzeigeTyp( ClientDavInterface connection, Array array, RegelnProAnzeigeTyp regelnProAnzeigeTyp )
  {
    HashMap<SystemObject, ListeMitRegeln> para = regelnProAnzeigeTyp.getParameter ();

    // Feststellen wieviele Anzeigentypen versorgt sind
    
    int anzahl = 0;
    Iterator<Entry<SystemObject, ListeMitRegeln>> it = para.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry<SystemObject, ListeMitRegeln> entry = it.next();
      
      SystemObject wzgTyp = (SystemObject) entry.getKey();
      
      DataModel dataModel = connection.getDataModel();
      ConfigurationObject co = (ConfigurationObject) dataModel.getObject (wzgTyp.getPid());
      if (co != null)
        anzahl++;
    }
    
    // Anzeigetypen speichern
    
    array.setLength( anzahl );

    int index = 0;
    it = para.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry<SystemObject, ListeMitRegeln> entry = it.next();
      
      SystemObject   wzgTyp = (SystemObject) entry.getKey();
      ListeMitRegeln listeMitregeln = (ListeMitRegeln) entry.getValue();
      
      DataModel dataModel = connection.getDataModel();
      ConfigurationObject co = (ConfigurationObject) dataModel.getObject (wzgTyp.getPid());
      if (co == null)
        continue;

      Data data = array.getItem( index++ );
      
      System.out
          .println("VerwaltungParaPriorisierung.speichereParaProAnzeigeTyp() " + co.getPid());
      
      data.getReferenceValue( "AnzeigeTyp" ).setSystemObject( co );
      Array prioritaeten = data.getArray( "Prioritten" );
      
      speichereParaListeMitregeln (connection, listeMitregeln, prioritaeten);
    }
  }

  /**
   * Methode speichert eine Liste mit Regeln im DaV
   * @param listeMitregeln zu speichernde Daten
   * @param array Array in das die Daten geschrieben werden sollen
   */
  private void speichereParaListeMitregeln(ClientDavInterface connection, ListeMitRegeln listeMitregeln, Array array )
  {
    HashMap<WVZInhalt, RegelPriorisierung> regeln = listeMitregeln.getRegeln();
    
    // Feststellen wieviele WVZ-Inhalte versorgt sind
    
    int anzahl = 0;
    Iterator<Entry<WVZInhalt, RegelPriorisierung>> it = regeln.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry<WVZInhalt, RegelPriorisierung> entry = it.next();
      
      WVZInhalt inhalt = (WVZInhalt) entry.getKey();
      
      if (inhalt != null && inhalt.getPid() != null)
      {
	      DataModel dataModel = connection.getDataModel();
	      ConfigurationObject co = (ConfigurationObject) dataModel.getObject (inhalt.getPid());
	      if (co != null)
	        anzahl++;
      }
    }
    
    // WVZ-Inhalte speichern
    
    array.setLength( anzahl );
    
    int index = 0;
    it = regeln.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry<WVZInhalt, RegelPriorisierung> entry = it.next();
      
      WVZInhalt inhalt = (WVZInhalt) entry.getKey();
      RegelPriorisierung regelPriorisierung = (RegelPriorisierung) entry.getValue();
      
      if (inhalt == null)
      	continue;
      
      if (inhalt.getPid() == null)
      	continue;
      	
      
      DataModel dataModel = connection.getDataModel();
      ConfigurationObject co = (ConfigurationObject) dataModel.getObject (inhalt.getPid());
      if (co == null)
        continue;

      Data data = array.getItem( index++ );
      
      Data d1 = data.getItem( "Zeichenfolge" );
      inhalt.erzeugeAttributgruppe ((ClientDavConnection) connection, d1);
      
      Data d2 = data.getItem( "Prioritten" );
      
      speichereParaRegelPriorisierung (connection, regelPriorisierung, d2);
    }
  }

  /**
   * Methode speichert die Regeln Priorisierung im DaV
   * @param connection 
   * @param regelPriorisierung zu speichernde Daten
   * @param data Item im DaV
   */
  private void speichereParaRegelPriorisierung( ClientDavInterface connection, RegelPriorisierung regelPriorisierung, Data data )
  {
    data.getUnscaledValue( "Prioritt" ).set( regelPriorisierung.getPrioritaet() );
    
    HashMap<Aspect, Integer> prioritaeten = regelPriorisierung.getPrioritaetAspekte();
    
    // Feststellen wieviele Aspekte versorgt sind
    
    int anzahl = 0;
    Iterator<Entry<Aspect, Integer>> it = prioritaeten.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry<Aspect, Integer> entry = it.next();
      
      Aspect aspekt = (Aspect) entry.getKey();
      
      DataModel dataModel = connection.getDataModel();
      ConfigurationObject co = (ConfigurationObject) dataModel.getObject (aspekt.getPid());
      if (co != null)
        anzahl++;
    }
    
    Array array = data.getArray( "Schaltgrnde" );
    array.setLength( anzahl );

    // Aspekte speichern 
    
    int index = 0;
    it = prioritaeten.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry<Aspect, Integer> entry = it.next();
      
      Aspect aspekt = (Aspect) entry.getKey();
      Integer prio  = (Integer) entry.getValue();
      
      DataModel dataModel = connection.getDataModel();
      ConfigurationObject co = (ConfigurationObject) dataModel.getObject (aspekt.getPid());
      if (co == null)
        continue;

      Data data1 = array.getItem( index++ );
      
      data1.getReferenceValue( "Schaltgrund" ).setSystemObject( co );
      data1.getUnscaledValue( "Prioritt" ).set( prio );
    }
  }
  
  /**
   * Methode liefert die Parameter als Attributgruppe fr den DaV
   * @return Attributgruppe
   */
  public Data getData ()
  {
  	return erzeugeAttributgruppe();
  }

  /**
   * Methode liefert die Variable m_parameterAnlage zurck
   * @return Variable m_parameterAnlage
   */
  public RegelnProAnzeigeTyp getParameterAnlage()
  {
    return m_parameterAnlage;
  }

  /**
   * Methode liefert die Parameter zu einer bestimmten Pid
   * @param pid Pid des gesuchten Objekts, dies kann die Pid der SBA, die Pid eines Anzeigequerschnitts oder
   * die Pid einer Anzeige sein.
   * @return Parameter zu der Pid
   */
	public RegelnProAnzeigeTyp getParameter(String pid)
  {
    /*
     * Parameter Anlage
     */
    if (pid.equals(_pidSBA))
      return getParameterAnlage();
    
    /*
     * Parameter Anzeigequerschnitt
     */
    for (SystemObject so : m_parameterProAnzeigeQuerschnitt.keySet())
    {
      if (pid.equals(so.getPid()))
        return m_parameterProAnzeigeQuerschnitt.get(so);
    }
    
    /*
     * Parameter Anzeige
     */
    for (SystemObject so : m_parameterProAnzeige.keySet())
    {
      if (pid.equals(so.getPid()))
        return m_parameterProAnzeige.get(so);
    }
    
    /*
     * Parametersatz noch nicht definiert
     */
   
    return new RegelnProAnzeigeTyp();
  }

  /**
   * Methode bernimmt die Parameter fr eine bestimmte Pid
   * @param pid Pid
   * @param regeln Regeln
   */
	public void setParameter(String pid, RegelnProAnzeigeTyp regeln)
  {
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    /*
     * Parameter Anlage
     */
    if (pid.equals(_pidSBA))
    {
      m_parameterAnlage = regeln;
    }
    else if (connection != null)
    {
    	/*
    	 * Prfen ob Anzeigequerschnitt oder Anzeige
    	 */
    	
    	SystemObject so = connection.getDataModel().getObject(pid);
    	if (so != null)
    	{
    		if (so.getType().getPid().equals("typ.anzeigeQuerschnitt"))
    			m_parameterProAnzeigeQuerschnitt.put(so, regeln);

    		if (so.getType().getPid().equals("typ.anzeige"))
    			m_parameterProAnzeige.put(so, regeln);
    	}
    }
    
    benachrichtigeListener();
  }

	/**
   * Methode liefert parameterProAnzeigeQuerschnitt zurck
   * @return parameterProAnzeigeQuerschnitt
   */
  public HashMap<SystemObject, RegelnProAnzeigeTyp> getParameterProAnzeigeQuerschnitt()
  {
  	return m_parameterProAnzeigeQuerschnitt;
  }

	/**
   * Methode liefert parameterProAnzeige zurck
   * @return parameterProAnzeige
   */
  public HashMap<SystemObject, RegelnProAnzeigeTyp> getParameterProAnzeige()
  {
  	return m_parameterProAnzeige;
  }

}
