/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Timer;
import java.util.TimerTask;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.Status;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.sts.buv.plugin.workflow.tsf.Activator;

/**
 * Liefert Informationen ber die aktiven Workflows fr die Darstellung im GUI.
 * 
 * @author HofmannJ
 * 
 */
public class GUIModelTSFWorkflow
{
	/**
	 * Key=Objekt Typ <code>typ.temporaereSeitenstreifenFreigabe</code>,
	 * Value=Java Reprsentation.
	 */
	private TreeMap<String, M_TSFWorkflow> _tm = new TreeMap<String, M_TSFWorkflow>();

	/**
	 * Listenersupport
	 */
	private PropertyChangeSupport _pcs = new PropertyChangeSupport(this);

	/**
	 * Wird gesendet mit einem Array aller registrierten {@link M_TSFWorkflow}s
	 * als newValue, sobald sich ein Listener anmeldet.
	 */
	public static final String PROP_INIT = "PROP_INIT";

	/**
	 * Wird gesendet, sobald ein {@link M_TSFWorkflow} hinzugefgt. Falls vorher
	 * ein alter verfgbar war wird dieser als oldValue bermittelt, der neue als
	 * newValue.
	 */
	public static final String PROP_ADDED = "PROP_ADDED";

	/**
	 * Wird gesendet, sobald ein {@link M_TSFWorkflow} entfernt wird. Der oldValue
	 * entspricht dem entfernten {@link M_TSFWorkflow}, der newValue ist
	 * <code>null</code>.
	 */
	public static final String PROP_REMOVED = "PROP_REMOVED";

	private Timer _timer;

	public GUIModelTSFWorkflow()
	{
		starteTimerGueltigkeitenPruefung();
	}

	public void dispose()
	{
		stoppeTimerGueltigkeitenPruefung();
	}

	protected void putWorkflow(M_TSFWorkflow workflow)
	{
//		System.err.println("Workflow empfangen -> " + workflow.toString());
		
		if ((workflow != null) && (workflow.getSo() != null))
		{
			if (!workflow.sindAlleZustaendeErreicht())
			{
				M_TSFWorkflow oldVal = null;
				M_TSFWorkflow newVal = workflow;

				synchronized (_tm)
				{
					if (_tm.containsKey(workflow.getSo().getPid()))
					{
						oldVal = _tm.get(workflow.getSo().getPid());
					}
					_tm.put(workflow.getSo().getPid(), workflow);
				}

				_pcs.firePropertyChange(PROP_ADDED, oldVal, newVal);
			} else
			{
				/*
				 * Alle geforderten Zustnde der Programme des Workflows sind erreicht,
				 * es ist nicht notwendig diesen noch darzustellen.
				 */
				// ILog logger = Activator.getDefault().getLog();
				// logger.log(new Status(Status.INFO, Activator.PLUGIN_ID,
				// "Alle Programme des nachfolgenden Workflows wurden geschalten: "
				// + workflow.getSo().getPidOrNameOrId()
				// + ". Dieser alte Workflow wird nicht mehr dargestellt."));

				/*
				 * WF entfernen.
				 */
				removeWorkflow(workflow.getSo());
			}
			
		} else
		{
			System.err.println("Workflow oder dessen SystemObject ist null.");
		}

		pruefeTimerStartBedingung();
	}

	/**
	 * Timer fr Workflowprfung starten
	 */
	private void pruefeTimerStartBedingung()
	{
		boolean mindestensEinWorkflowVorhanden = false;

		synchronized (_tm)
		{
			if (_tm.size() > 0)
			{
				mindestensEinWorkflowVorhanden = true;
			}
		}

		if (mindestensEinWorkflowVorhanden)
		{
			starteTimerGueltigkeitenPruefung();
		} else
		{
			stoppeTimerGueltigkeitenPruefung();
		}

	}

	protected M_TSFWorkflow removeWorkflow(SystemObject so)
	{
		M_TSFWorkflow removedWF = null;

		if (so != null)
		{
			removedWF = removeWorkflow(so.getPid());
		}

		return removedWF;
	}

	protected M_TSFWorkflow removeWorkflow(String pid)
	{
		M_TSFWorkflow removedWF = null;

		if (pid != null)
		{
			synchronized (_tm)
			{
				if (_tm.containsKey(pid))
				{
					removedWF = _tm.remove(pid);
				}
			}

			if (removedWF != null)
			{
				_pcs.firePropertyChange(PROP_REMOVED, removedWF, null);
			}
		}

		pruefeTimerStartBedingung();

		return removedWF;
	}

	protected M_TSFWorkflow[] getWorkflows()
	{
		LinkedList<M_TSFWorkflow> lWF = new LinkedList<M_TSFWorkflow>();

		synchronized (_tm)
		{
			Iterator<Entry<String, M_TSFWorkflow>> it = _tm.entrySet().iterator();
			while (it.hasNext())
			{
				Entry<String, M_TSFWorkflow> entry = it.next();
				if (entry.getValue() != null)
				{
					lWF.add(entry.getValue());
				}
			}
		}

		return lWF.toArray(new M_TSFWorkflow[lWF.size()]);
	}

	/**
	 * Liefert eien {@link M_TSFWorkflow} an Hand seiens SystemObjekte vom Typ
	 * <code>typ.temporaereSeitenstreifenFreigabe</code> aus der Map.
	 * 
	 * @param so
	 *          SystemObjekt vom Typ
	 *          <code>typ.temporaereSeitenstreifenFreigabe</code>.
	 * @return {@link M_TSFWorkflow} oder <code>null</code> wenn nicht verfgbar.
	 */
	protected M_TSFWorkflow getWorkflow(SystemObject so)
	{
		M_TSFWorkflow re = null;

		if (so != null)
		{
			synchronized (_tm)
			{
				re = _tm.get(so.getPid());
			}
		}

		return re;
	}

	public void addListener(PropertyChangeListener pcl)
	{
		_pcs.addPropertyChangeListener(pcl);

		PropertyChangeEvent pce = new PropertyChangeEvent(this, PROP_INIT, null,
				getWorkflows());
		pcl.propertyChange(pce);
	}

	public void removeListener(PropertyChangeListener pcl)
	{
		_pcs.removePropertyChangeListener(pcl);
	}

	/**
	 * Prft ob bei den vorhandenen Workflows bereits eine Abarbeitung geschehen
	 * ist. Ist dies der Fall, wird der Workflow entfernt, der Dialog mit den
	 * Schaltempfehlungen verschwindet.
	 */
	private void pruefeWorflowGueltigkeiten()
	{
		M_TSFWorkflow[] aWF = getWorkflows();

		if ((aWF != null) && (aWF.length > 0))
		{
			for (M_TSFWorkflow tsfWorkflow : aWF)
			{
				if (tsfWorkflow != null)
				{
					if (tsfWorkflow.sindAlleZustaendeInBearbeitung())
					{
						/*
						 * Hier wird ein Workflow disposed, dessen Ziel-Zustnde erreicht
						 * wurden.
						 */
            System.out.println("DaVKomTSFWorkflow.update(REM5)");
						removeWorkflow(tsfWorkflow.getSo());
					}
				}
			}
		}
	}

	private void starteTimerGueltigkeitenPruefung()
	{
		if (_timer == null)
		{
			// logge(Status.INFO, "Starte Timer WF Gueltigkeitenpruefung.");
			TimerTask task = new TimerTask()
			{

				@Override
				public void run()
				{
					pruefeWorflowGueltigkeiten();
				}
			};

			_timer = new Timer("Pruefung WF Gueltigkeiten");
			_timer.schedule(task, 0l, 5l * 1000l);
		}
	}

	private void stoppeTimerGueltigkeitenPruefung()
	{
		if (_timer != null)
		{
			// logge(Status.INFO,
			// "Stoppe Timer WF Gueltigkeitenpruefung -> Keine WF zum berwachen vorhanden.");
			_timer.cancel();
			_timer = null;
		}
	}

	private void logge(int sev, String msg)
	{
		ILog logger = Activator.getDefault().getLog();
		logger.log(new Status(sev, Activator.PLUGIN_ID, msg));
	}

}
